@@ -4,7 +4,7 @@
 /*           http://open-jtalk.sourceforge.net/                      */
 /* ----------------------------------------------------------------- */
 /*                                                                   */
-/*  Copyright (c) 2008-2011  Nagoya Institute of Technology          */
+/*  Copyright (c) 2008-2012  Nagoya Institute of Technology          */
 /*                           Department of Computer Science          */
 /*                                                                   */
 /* All rights reserved.                                              */
@@ -71,7 +71,7 @@ NJD_SET_ACCENT_TYPE_C_START;
 
 #define MAXBUFLEN 1024
 
-static char get_token_from_string(char *str, int *index, char *buff)
+static char get_token_from_string(const char *str, int *index, char *buff)
 {
    char c;
    int i = 0;
@@ -89,7 +89,7 @@ static char get_token_from_string(char *str, int *index, char *buff)
    return c;
 }
 
-static void get_rule(char *input_rule, char *prev_pos, char *rule, int *add_type)
+static void get_rule(const char *input_rule, const char *prev_pos, char *rule, int *add_type)
 {
    int index = 0;
    char buff[MAXBUFLEN];
@@ -132,6 +132,9 @@ void njd_set_accent_type(NJD * njd)
    int add_type = 0;
    int mora_size = 0;
 
+   if (njd == NULL || njd->head == NULL)
+      return;
+
    for (node = njd->head; node != NULL; node = node->next) {
       if (NJDNode_get_string(node) == NULL)
          continue;
